/* Copyright (C) 1989 Aladdin Enterprises.  All rights reserved.
   Distributed by Free Software Foundation, Inc.

This file is part of Ghostscript.

Ghostscript is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the Ghostscript General Public License for full details.

Everyone is granted permission to copy, modify and redistribute
Ghostscript, but only under the conditions described in the Ghostscript
General Public License.  A copy of this license is supposed to have been
given to you along with Ghostscript so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.  */

/* zmisc.c */
/* Miscellaneous operators for GhostScript */
#include "ghost.h"
#include "errors.h"
#include "oper.h"
#include "dict.h"
#include "store.h"

/* Import the dictionary stack pointer (for bind) */
extern ref *dsp;

/* Import the operand stack limit */
extern ref *ostop;

/* bind */
int
zbind(register ref *op)
{	ref *bsp = op;			/* bottom of stack */
	/****** WHAT ABOUT PACKED ARRAYS? ******/
	/****** SHOULD MAKE INNER ARRAYS READ-ONLY ******/
	check_type(*op, t_array);
	if ( !(r_attrs(op) & a_write) ) return e_invalidaccess;
	*++bsp = *op;
	while ( bsp > op )
	   {	while ( bsp->size )
		   {	ref *tp = bsp->value.refs++;
			bsp->size--;
			switch ( r_type(tp) )
			   {
			case t_name:	/* bind the name if an operator */
				if ( r_attrs(tp) & a_executable )
				   {	ref *pvalue = dict_search(tp, dsp);
					if ( pvalue != 0 && r_btype(pvalue) == t_operator && (r_attrs(pvalue) & a_executable) )
						store(tp, *pvalue);
				   }
				break;
			case t_array:	/* push into array if procedure */
				if ( !(~r_attrs(tp) & (a_executable+a_write)) && bsp < ostop )
					*++bsp = *tp;
			   }
		   }
		bsp--;
	   }
	return 0;
}

/* currenttime */
int
zcurrenttime(register ref *op)
{	long date_time[2];
	gs_get_clock(date_time);
	push(1);
	make_real(op, date_time[0] * 1440.0 + date_time[1] / 60000.0);
	return 0;
}

/* setdebug */
int
zsetdebug(register ref *op)
{	int i;
	check_type(op[-1], t_string);
	check_type(*op, t_boolean);
#ifdef DEBUG
	for ( i = 0; i < op[-1].size; i++ )
		Debug[op[-1].value.bytes[i] & 127] = op->value.index;
#endif
	pop(2);
	return 0;
}

/* usertime */
int
zusertime(register ref *op)
{	long date_time[2];
	gs_get_clock(date_time);
	push(1);
	make_int(op, date_time[0] * 86400000L + date_time[1]);
	return 0;
}

/* ------ Initialization procedure ------ */

void
zmisc_op_init()
{	static op_def my_defs[] = {
		{"1bind", zbind},
		{"0currenttime", zcurrenttime},
		{"2setdebug", zsetdebug},
		{"0usertime", zusertime},
		op_def_end
	};
	z_op_init(my_defs);
}
